import { URL } from 'url';
import ResponseBuilder from './ResponseBuilder.js'
import { getContentTypeFrom }  from '../scripts/contentTypeUtil.js';

const BASE = 'http://localhost/';

export default class RequestController {

  #request;
  #response;
  #url;

  constructor(request, response) {
    this.#request = request,
    this.#response = response;
    this.#url = new URL(this.request.url,BASE).pathname;   // on ne considère que le "pathname" de l'URL de la requête
  }

  get response() {
    return this.#response;
  }
  get request() {
    return this.#request;
  }
  get url() {
    return this.#url;
  }

  async handleRequest() {
    this.response.setHeader("Content-Type" , getContentTypeFrom(this.url) || 'text/html');
    await this.buildResponse();
    this.response.end();
  }

  async buildResponse()  {
    await new ResponseBuilder(this.url, this.#response).handleResponse();
  }

}