import * as fs from 'fs/promises';
import ErrorResponseBuilder from './ErrorResponseBuilder.js';
import { URL } from 'url';

export default class ResponseBuilder {
    #request;
    #response;
    #url;
    
    constructor(url, response) {
        this.#response =response; 
        this.#url = url;
    }

    get url() {
      return this.#url;
    }

    get response() {
      return this.#response;
    }

    get request() {
      return this.#request;
    }

    async handleResponse() {
        await this.buildResponse();
      }
      
    async buildResponse()  {    
        
        // routage "à la main"
        if (this.url === '/' ){
          await this.readResponse('./public/index.html');
        }
        else if (this.url.startsWith('/about')){
          await this.readResponse('./public/html/about.html');
        }
        else if(this.url.startsWith('/auctioneer')) {
          await this.readResponse('./public/html/auctioneer.html');
        }
        else if(this.url.startsWith('/bidder')) {
          await this.readResponse('./public/html/bidder.html');
        }
        else {
          await this.readResponse(`./public${this.#url}`);
        }
      }      
    
      async readResponse(file)  {
        try {
          // check if resource is available
          await fs.access(file);
          // read the requested resource content
          const data = await fs.readFile(file);
          // send resource content
          this.response.statusCode = 200;
          this.response.write(data);
        }
        catch(err) { // resource is not available
          new ErrorResponseBuilder(this.url, this.response, 404).handleError(); 
        }
      }
    

}