export default class IOController {
    #io;
    #clients;
  
    constructor(io) {
      this.#io = io;
      this.#clients = new Map();
    }
  
    registerSocket(socket) {
      console.log(`new connection with id ${socket.id}`);
      this.setupListeners(socket);
    }
  
    setupListeners(socket) {
      socket.on("auctioneer", ()=> this.registerAuctioneer(socket));
      socket.on( 'infos', (name, value) => this.transmitName(name, value) );
      socket.on("prixActuel", (prixActuel, selectedPrix) => this.transmitPrixActuel(prixActuel, selectedPrix, socket.id));
      socket.on( 'disconnect' , () => this.leave(socket) );
      socket.on("adjuge", ()=> this.saleProduct());
      socket.on("encherisseur", () => this.registerBidder(socket));
    }

    registerAuctioneer(socket) {
      if (this.#clients.has("auctioneer")) {
        socket.emit("alreadyAuct");
      }
      else {
        this.#clients.set("auctioneer", socket);
      }
    }

    registerBidder(socket) {
      this.#clients.set("bidder", socket);
    }
  
    transmitName(name, value) {
      this.#io.emit("infos", name, value);
    }

    transmitPrixActuel(prixActuel, selectedPrix, sock) {
      this.#io.emit("changePrix", prixActuel, selectedPrix, sock);
      this.#clients.get("bidder").emit("enchere", selectedPrix);
    }

    saleProduct() {
      this.#io.emit("adjuge");
      this.#clients.get("bidder").emit("winner");
    }
  
    leave(socket) {
      console.log(`disconnection from ${socket.id}`);
      if (this.#clients.get("auctioneer") === socket){
        this.#clients.delete("auctioneer");
        this.#io.emit("deconnexion");
      }
    }
    
}
  