var createError = require('http-errors');
var express = require('express');
var path = require('path');
var cookieParser = require('cookie-parser');
var logger = require('morgan');

var indexRouter = require('./routes/index');
const jsonRouter = require('./routes/jsonRouter');
const taskRouter = require('./routes/taskRouter');

// define middlewares managing unknown routes
const error = require('./middlewares/error.middleware');

// mongoDb
const dbConnection = require('./controllers/db.controller.js');

var app = express();

// view engine setup
app.set('views', path.join(__dirname, 'views'));
app.set('view engine', 'pug');

app.use(logger('dev'));
app.use(express.json());
app.use(express.urlencoded({ extended: false }));
app.use(cookieParser());
app.use(express.static(path.join(__dirname, 'public')));

app.use('/', indexRouter);
app.use('/json', jsonRouter);
app.use('/task', taskRouter);
//app.use('/mongodb://username:password@host:port/tasksBase', mongoRouter);



// catch 404 and forward to error handler
app.use(function(req, res, next) {
  next(createError(404));
});

// error handler
app.use(error);

module.exports = app;
