# Auteurs

### BAH Mamadu Lamarana

# Projet Showtime

construire une application qui permet la gestion de spectacles et la réservation de tickets par ses utilisateurs. L'application se décompose en deux parties. La première concerne la partie administration de l'application et la seconde est pour les utilisateurs.

# Documentation de l'API REST de l'application

L'API REST de cette application offre plusieurs routes pour effectuer des opérations sur les données côté serveur. Chaque route est associée à une méthode HTTP (GET, PUT, DELETE) et correspond à une fonction dans le contrôleur.

**Pour un Admin :**
    créer un spectacle en en donnant une description et en précisant le nombre de places disponibles,
    visualiser la liste des spectacles déjà créés,
    supprimer un spectacle. Lors de la suppression d'un spectacle, les tickets réservés par les clients (voir ci-dessous) doivent être supprimés.

**Pour un User :**
    consulter la liste des spectacles déjà créés,
    réserver des tickets pour un spectacle,
    visualiser la listes des tickets réservés,
    annuler les réservations pour un spectacle.


## Routes disponibles pour un client (non administrateur)

### 1. Obtenir les informations de l'utilisateur connecté
- **Route :** `GET /me`
- **Fonction :** `indexController.me`
- **Description :** Cette route permet à l'application d'obtenir les informations de l'utilisateur connecté.
- **Données de la requête :** Aucune donnée n'est transmise dans la requête.
- **Données de la réponse :** Les informations de l'utilisateur connecté sont renvoyées, notamment son nom et son statut d'administrateur.
- **Utilisation de la réponse :** L'application utilise les informations renvoyées pour afficher les détails de l'utilisateur connecté.

### 2. Obtenir la liste des spectacles
- **Route :** `GET /items`
- **Fonction :** `indexController.listShow`
- **Description :** Cette route permet à l'application d'obtenir la liste des spectacles disponibles.
- **Données de la requête :** Aucune donnée n'est transmise dans la requête.
- **Données de la réponse :** La liste des spectacles est renvoyée.
- **Utilisation de la réponse :** L'application utilise la liste des spectacles pour afficher les options disponibles pour les utilisateurs.

### 3. Obtenir les billets réservés par l'utilisateur connecté
- **Route :** `GET /tickets`
- **Fonction :** `indexController.listTickets`
- **Description :** Cette route permet à l'application d'obtenir les billets réservés par l'utilisateur connecté.
- **Données de la requête :** Aucune donnée n'est transmise dans la requête.
- **Données de la réponse :** Les billets réservés par l'utilisateur sont renvoyés.
- **Utilisation de la réponse :** L'application utilise les billets réservés pour afficher les détails des réservations de l'utilisateur.

### 4. Mettre à jour les informations d'un billet réservé
- **Route :** `PUT /tickets`
- **Fonction :** `indexController.update`
- **Description :** Cette route permet à l'application de mettre à jour les informations d'un billet réservé par l'utilisateur connecté.
- **Données de la requête :** Les nouvelles informations du billet sont transmises dans le corps de la requête au format JSON.
- **Données de la réponse :** Un message de confirmation de la mise à jour est renvoyé.
- **Utilisation de la réponse :** L'application affiche un message confirmant que la mise à jour du billet a été effectuée avec succès.

### 5. Supprimer un billet réservé par l'utilisateur connecté
- **Route :** `DELETE /:showId`
- **Fonction :** `indexController.deleteTickets`
- **Description :** Cette route permet à l'application de supprimer un billet réservé par l'utilisateur connecté.
- **Données de la requête :** L'identifiant du spectacle (`showId`) est transmis dans la requête.
- **Données de la réponse :** L'identifiant du spectacle supprimé est renvoyé.
- **Utilisation de la réponse :** L'application affiche un message indiquant que le billet a été supprimé avec succès.

### 6. Page "À propos"
- **Route :** `GET /about`
- **Fonction :** `indexController.about`
- **Description :** Cette route renvoie une page statique "À propos".
- **Données de la requête :** Aucune donnée n'est transmise dans la requête.
- **Données de la réponse :** La page "À propos" est renvoyée.
- **Utilisation de la réponse :** L'application affiche la page "À propos" pour fournir des informations sur l'application.

### 7. Page "Admin uniquement"
- **Route :** `GET /adminonly`
- **Fonction :** `indexController.adminonly`
- **Description :** Cette route renvoie une page statique accessible uniquement aux administrateurs.
- **Données de la requête :** Aucune donnée n'est transmise dans la requête.
- **Données de la réponse :** La page "Admin uniquement" est renvoyée.
- **Utilisation de la réponse :** L'application affiche la page "Admin uniquement" uniquement si l'utilisateur connecté est un administrateur.

## Routes disponibles pour l'administration

### 1. Page d'accueil de l'administration
- **Route :** `GET /`
- **Fonction :** `adminController.home`
- **Description :** Redirige vers la page d'accueil de l'administration (`/admin.html`).
- **Données de la requête :** Aucune donnée n'est transmise dans la requête.
- **Données de la réponse :** Une redirection vers `/admin.html`.

### 2. Lister les spectacles
- **Route :** `GET /items`
- **Fonction :** `adminController.list`
- **Description :** Permet d'obtenir la liste des spectacles disponibles.
- **Middleware :** `authMiddleware.validToken`, `authMiddleware.isAdmin`
- **Données de la requête :** Aucune donnée spécifique n'est requise.
- **Données de la réponse :** Une liste de tous les spectacles disponibles.
- **Utilisation de la réponse :** Affichage de la liste des spectacles dans l'interface d'administration.

### 3. Créer un spectacle
- **Route :** `POST /items`
- **Fonction :** `adminController.create`
- **Description :** Permet de créer un nouveau spectacle.
- **Middleware :** `authMiddleware.validToken`, `authMiddleware.isAdmin`
- **Données de la requête :** Les informations du nouveau spectacle (`description`, `places`) au format JSON dans le corps de la requête.
- **Données de la réponse :** Les détails du spectacle nouvellement créé.
- **Utilisation de la réponse :** Confirmation de la création du spectacle et mise à jour de la liste des spectacles dans l'interface d'administration.

### 4. Supprimer un spectacle
- **Route :** `DELETE /:showId`
- **Fonction :** `adminController.deleteShow`
- **Description :** Permet de supprimer un spectacle existant.
- **Middleware :** `authMiddleware.validToken`, `authMiddleware.isAdmin`
- **Données de la requête :** L'identifiant du spectacle (`showId`) est transmis dans l'URL.
- **Données de la réponse :** Aucune donnée spécifique n'est renvoyée, seulement une confirmation de la suppression.
- **Utilisation de la réponse :** Confirmation de la suppression du spectacle et mise à jour de la liste des spectacles dans l'interface d'administration.

## Authentification et autorisation

Les routes d'administration nécessitent une authentification (`authMiddleware.validToken`) et vérifient si l'utilisateur est administrateur (`authMiddleware.isAdmin`) pour autoriser l'accès aux opérations d'administration.

## Utilisation de l'API côté client

Les requêtes `fetch` sont utilisées dans le client d'administration pour interagir avec l'API. Les opérations incluent l'obtention de la liste des spectacles, la création d'un nouveau spectacle, et la suppression d'un spectacle existant.


Cette documentation décrit les différentes routes disponibles dans l'API REST de l'application, ainsi que les opérations associées aux données effectuées par le serveur.

# Commandes

## Préparation du projet

Pour mettre en place l'environnement d'execution de l'application, on se place dans le dossier de l'application **showtime**.
ensuite on va d'abord dans le dossier **client** (respectivement dans le dossier **server**) pour insataller les modules nodes nécessaires,
on exécute :

    npm install

Vu que j'ai utilisé webpack. il faut aussi génerer les fichiers clients du dossier server/public, il faut se placer dans le dossier **client**
et exécutér cette commande ci-dessous pour produire les fichiers clients nécéssaires du dossier au **server/public**.


    npm run build

ensuite se placer dans le dossier **/server**, créer un dossier **dbData** pour accueillir les données du serveur,
et ensuite éxecuter la commande ci-dessous pour demarrer la base de données.

    mongod --dbpath mongod dbData

    (NB : bien veiller à, si dossier dbData inéxistant et que mongodb ne le crée pas, créer manuellement (dbData)).

esuite on peut demarrer le serveur afin de pouvoir se connecter avec la commande :
    npm run start où (nodemon)

Le serveur se lancera (un message s'affichera dans le temrinal) et vous pourrez vous connecter au site de showtime à l'adresse suivante dans votre navigateur préferée:

    localhost:3000/

## Importer Document spectacle:
    avce le fichier se trouvant au server/misc/shows.json, on peut initialiser la base de données avec ces spectacles. pour cela, il faut utiliser
    mongoimport comme suit :
    se placer dans le dossier ./server et faire

    mongoimport --db booksBase --collection shows --file ./misc/shows.json
    

# Précisions
## Authentification JWT:
    Utilisation de Authentification JWT
    Dans notre application, nous avons implémenté un système d'authentification basé sur les JSON Web Tokens (JWT) pour sécuriser l'accès aux
    routes sensibles de notre API. JWT est une méthode standard pour réaliser une authentification stateless entre le client et le serveur. Ce
    système permet à l'utilisateur de se connecter une fois et d'utiliser un token pour s'authentifier lors des requêtes suivantes.

## Connexion et enregistrement en tant que admin ou user:
    pour s'enregistrer en tant que admin, il faut utiliser le boutton registerAsAdmin où avoir un login qui est "admin".
    pour s'enregistrer en tant que user simple, il faut utiliser le boutton register.
    NB : il ne peut y avoir qu'un seul admin.

## Extensions
    pas d'extensions mis en place.




