import { URL } from 'url';
import FirstBuilder from './FirstBuilder.js';
import SecondBuilder from './SecondBuilder.js';
import JsonResponseBuilder from './JsonResponseBuilder.js';
import ErrorResponseBuilder from './ErrorResponseBuilder.js';
import RandomJsonBuilder from './RandomJsonBuilder.js';
import FileResponseBuilder from './FileResponseBuilder.js';

export default class ResponseBuilder {
    request;
    response;
    url;
    
    constructor(request, response) {
        this.request =request,
        this.response =response; 
        this.url = new URL(request.url, `http://${request.headers.host}`);      }

      handleResponse() {
        this.prepareResponse();
        this.buildResponse();
      }
      
      prepareResponse() {
        this.response.statusCode = 200;
        this.response.setHeader( 'Content-Type' , 'text/html');
      } 

      buildResponse()  {    
        
        // routage "à la main"
        if (this.url.pathname.startsWith('/first')){
          const first = new FirstBuilder(this.request, this.response);
          first.bresponse();
          first.endResponse();
        }
        else if (this.url.pathname.startsWith('/second')) {
          const second = new SecondBuilder(this.request, this.response);
          second.bresponse();
          second.endResponse();
        }  
        else if (this.url.pathname.startsWith('/json')) {
          new JsonResponseBuilder(this.request, this.response).bResponse();
        }
        else if (this.url.pathname.startsWith('/random')) {
          new RandomJsonBuilder(this.request, this.response).bResponse();
        }
        else if (this.url.pathname.startsWith('/public')) {
          new FileResponseBuilder(this.request, this.response).hResponse();
        }
        else {
          new ErrorResponseBuilder(this.request, this.response, 404).handleError(); 
        }
      }      
    
}