let userlogin;
let userpassword;
let username;

const displayMessage = msg => document.getElementById('content').textContent = msg;
const resetDesc = () => document.getElementById('desc').value = '';

const setup = () => {
  username = document.getElementById('username');
  getUser();
  document.getElementById('create').addEventListener('click', createShow);
  displayList();
  document.getElementById('logout').addEventListener('click', logout);
  document.getElementById("desc").addEventListener('input', activeButton);
}

window.addEventListener('DOMContentLoaded', setup);

const getUser = async () => {
  const requestOptions = {
    method: 'GET',
  };
  const response = await fetch('/me', requestOptions);
  if (response.ok) {
    const user = await response.json();
    username.textContent = user.name;
  }
  else {
    const error = await response.json();
    handleError(error);
  }
}

// fetch GET all shows
const displayList = async () => {
  const requestOptions = {
    method: 'GET'
  };
  const response = await fetch('/admin/items', requestOptions)
  const allShows = await response.json();

  //(ici : code  exploitation de allShows)
  const list = document.getElementById('list');
  list.textContent = '';
  allShows.forEach(show => addToList(show, list));
}

const addToList = (show, list) => {
  const node = document.createElement('div');
  node.className = 'show';
  node.textContent = `${show.description} (${show.places})`;
  const deleteButton = document.createElement('button');
  deleteButton.className = 'delete';
  deleteButton.addEventListener('click', () => deleteShow(show._id));
  deleteButton.textContent = 'annuler';
  node.appendChild(deleteButton);
  //
  list.appendChild(node);
}

// fetch POST to create one show
const createShow =
  async () => {
    const newShowData = {
      description: document.getElementById('desc').value,
      places: document.getElementById('place').value,
    };
    const body = JSON.stringify(newShowData);
    const requestOptions = {
      method: 'POST',
      headers: { "Content-Type": "application/json" },
      body: body
    };
    const response = await fetch('/admin/items', requestOptions);
    if (response.ok) {
      const createdShow = await response.json();
      displayMessage(`${createdShow.description} créée`);
      resetDesc();
      displayList();
    }
    else {
      const errorMsg = await response.json();
      displayMessage(`problème création ${errorMsg}`);
    }
    document.getElementById('create').disabled = true;
  }

// fetch DELETE to delete one task with given show id
const deleteShow =
  async showId => {
    const requestOptions = {
      method: 'DELETE'
    };
    const response = await fetch(`/admin/${showId}`, requestOptions);
    if (response.ok) {
      displayMessage('suppression effectuée');
      displayList();
      await fetch(`/${showId}`, requestOptions);
    }
    else {
      displayMessage('problème suppression');
    }
  }

const logout = async () => {
  const requestOptions = {
    method: 'GET',
  };
  const response = await fetch(`/access/logout`, requestOptions);
  if (response.ok) {
    window.location.href = '/';
  }
}

const handleError = error => {
  if (error.redirectTo)
    window.location.href = error.redirectTo;
  else
    console.log(`erreur : ${error.message}`);
}

const activeButton = () => {
  document.getElementById('create').disabled = false;
}

